package com.bytesw.bytebot.bean.agency;

import com.bytesw.bytebot.model.agency.*;
import com.google.gson.annotations.Expose;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
public class FlightBean implements Serializable {

    @Expose
    private Integer id;

    @Expose
    private Integer airlineId;

    @Expose
    private String airlineName;

    @Expose
    private Integer aircraftId;

    @Expose
    private String aircraftName;

    @Expose
    private Integer airportOriginId;

    @Expose
    private String airportOriginName;

    @Expose
    private Integer airportDestinationId;

    @Expose
    private String airportDestinationName;

    @Expose
    private String departureDate;

    @Expose
    private String arrivalDate;

    @Expose
    private List<ScaleBean> scales;

    @Expose
    private List<CostBean> costs;

    public static FlightBean miniClone(Flight flight) {
        FlightBean flightBean = new FlightBean();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");
        flightBean.setId(flight.getId());
        flightBean.setAirlineId(flight.getAirline().getId());
        flightBean.setAirlineName(flight.getAirline().getName());
        flightBean.setAircraftId(flight.getAircraft().getId());
        flightBean.setAircraftName(flight.getAircraft().getName());
        flightBean.setAirportOriginId(flight.getAirportOrigin().getId());
        flightBean.setAirportOriginName(flight.getAirportOrigin().getName());
        flightBean.setAirportDestinationId(flight.getAirportDestination().getId());
        flightBean.setAirportDestinationName(flight.getAirportDestination().getName());
        flightBean.setDepartureDate(flight.getDepartureDate().format(formatter));
        flightBean.setArrivalDate(flight.getArrivalDate().format(formatter));

        List<Scale> scales = flight.getScales();
        List<ScaleBean> scalesBean = new ArrayList<>();
        for (Scale scale : scales) {
            scalesBean.add(ScaleBean.miniClone(scale));
        }

        List<Cost> costs = flight.getCosts();
        List<CostBean> costsBean = new ArrayList<>();
        for (Cost cost : costs) {
            costsBean.add(CostBean.miniClone(cost));
        }

        flightBean.setScales(scalesBean);
        flightBean.setCosts(costsBean);

        return flightBean;
    }
}
