package com.bytesw.bytebot.bean.agency;

import com.bytesw.bytebot.model.agency.Cost;
import com.bytesw.bytebot.model.agency.Flight;
import com.google.gson.annotations.Expose;
import lombok.Getter;
import lombok.Setter;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
public class FlightSearchBean {
    @Expose
    private Integer id;

    @Expose
    private String airline;

    @Expose
    private String aircraft;

    @Expose
    private String flightType;

    @Expose
    private String airportOrigin;

    @Expose
    private String airportDestination;

    @Expose
    private String departureDate;

    @Expose
    private String arrivalDate;

    @Expose
    private Double amount;

    @Expose
    private List<ScaleBean> scales;

    public static FlightSearchBean miniClone(Flight flight, LocalDateTime date) {
        FlightSearchBean searchBean = new FlightSearchBean();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");

        LocalDateTime departureDate = LocalDateTime.of
                (date.getYear(), date.getMonth(), date.getDayOfMonth(), flight.getScales().get(0).getDepartureDate().getHour(),
                        flight.getScales().get(0).getDepartureDate().getMinute(), flight.getScales().get(0).getDepartureDate().getSecond());

        LocalDateTime arrivalDate = departureDate;
        Duration duration = Duration.between(flight.getScales().get(0).getDepartureDate(), flight.getScales().get(flight.getScales().size() - 1).getArrivalDate());
        arrivalDate = arrivalDate.plus(duration);


        searchBean.setId(flight.getId());
        searchBean.setAirline(flight.getAirline().getName());
        searchBean.setAircraft(flight.getAircraft().getName());
        searchBean.setFlightType(flight.getScales().size() > 1 ? "With scales" : "Direct");
        searchBean.setDepartureDate(departureDate.format(formatter));
        searchBean.setAirportOrigin(flight.getAirportOrigin().getName());
        searchBean.setAirportDestination(flight.getAirportDestination().getName());
        searchBean.setArrivalDate(arrivalDate.format(formatter));
        searchBean.setAmount(getAmount(flight.getCosts(), date));
        searchBean.setScales(flight.getScales().stream().map(ScaleBean::miniClone).collect(Collectors.toList()));

        return searchBean;
    }

    public static Double getAmount(List<Cost> costs, LocalDateTime date) {

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");

        for (Cost cost : costs) {
            LocalDateTime departureDate = cost.getDepartureDate();
            LocalDateTime arrivalDate = cost.getArrivalDate();

            if ((date.isAfter(departureDate) && date.isBefore(arrivalDate))
                    || date.isEqual(departureDate))
                return cost.getAmount();
        }

        throw new RuntimeException(String.format("Cost not found for %s", date.format(formatter)));
    }
}
