package com.bytesw.bytebot.bean.agency;

import com.bytesw.bytebot.model.agency.Cost;
import com.bytesw.bytebot.model.agency.Reservation;
import com.bytesw.bytebot.model.agency.Scale;
import com.bytesw.bytebot.model.agency.enums.ReservationStatusEnum;
import com.bytesw.bytebot.model.agency.enums.TicketTypeEnum;
import com.bytesw.bytebot.model.agency.enums.TravelClassEnum;
import com.google.gson.annotations.Expose;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
public class ReservationBean implements Serializable {

    @Expose
    private Integer id;

    @Expose
    private String passenger;

    @Expose
    private String airportOrigin;

    @Expose
    private Integer airportOriginId;

    @Expose
    private String airportDestination;

    @Expose
    private Integer airportDestinationId;

    @Expose
    private String departureDate;

    @Expose
    private Integer passengerId;

    @Expose
    private Integer agentId;

    @Expose
    private String ticketType;

    @Expose
    private String travelClass;

    @Expose
    private String status;

    @Expose
    private Double amount;

    @Expose
    private List<ScaleBean> scales;

    @Expose
    private Integer flightId;

    @Expose
    private FlightSearchBean flightSearchBean;

    public static ReservationBean miniClone(Reservation reservation) {
        ReservationBean reservationBean = new ReservationBean();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");

        reservationBean.setId(reservation.getId());
        reservationBean.setPassenger(String.format("%s %s", reservation.getPassenger().getName(), reservation.getPassenger().getLastname()));
        reservationBean.setPassengerId(reservation.getPassenger().getId());
        reservationBean.setAirportOrigin(reservation.getFlight().getAirportOrigin().getName());
        reservationBean.setAirportOriginId(reservation.getFlight().getAirportOrigin().getId());
        reservationBean.setAirportDestination(reservation.getFlight().getAirportDestination().getName());
        reservationBean.setAirportDestinationId(reservation.getFlight().getAirportDestination().getId());
        reservationBean.setScales(getScalesBean(reservation.getFlight().getScales()));
        reservationBean.setFlightId(reservation.getFlight().getId());
        reservationBean.setDepartureDate(reservation.getDepartureDate().format(formatter));
        reservationBean.setTicketType(reservation.getTicketType().getName());
        reservationBean.setTravelClass(reservation.getTravelClass().getName());
        reservationBean.setFlightSearchBean(FlightSearchBean.miniClone(reservation.getFlight(), reservation.getDepartureDate()));
        if(reservation.getPayment() != null)
            reservationBean.setAmount(reservation.getPayment().getAmount());
        reservationBean.setStatus(reservation.getStatus().getName());
        reservationBean.setAgentId(reservation.getAgencyAgent().getId());

        return reservationBean;
    }

    private static List<ScaleBean> getScalesBean(List<Scale> scales) {
        return scales.stream().map(ScaleBean::miniClone)
                .collect(Collectors.toList());
    }
}
