package com.bytesw.bytebot.controller.agency;

import com.bytesw.bytebot.bean.agency.AgentBean;
import com.bytesw.bytebot.bean.agency.SignUpBean;
import com.bytesw.bytebot.bean.agency.UpdatePasswordBean;
import com.bytesw.bytebot.service.agency.AgencyAgentService;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/service/agents")
public class AgencyAgentController {

    private final AgencyAgentService agentService;

    public AgencyAgentController(AgencyAgentService agentService) {
        this.agentService = agentService;
    }

    @GetMapping
    public ResponseEntity<List<AgentBean>> getAll() {
        return ResponseEntity.ok(agentService.getAll());
    }

    @PostMapping("/page")
    public ResponseEntity<CustomPaginationBean<AgentBean>> getByPagination(@RequestBody CustomPaginationBean<AgentBean> pagination) {
        return ResponseEntity.ok(agentService.getByPagination(pagination));
    }

    @GetMapping("{id}")
    public ResponseEntity<AgentBean> getById(@PathVariable Integer id) {
        return ResponseEntity.ok(agentService.getById(id));
    }

    @PostMapping("/create")
    public ResponseEntity<AgentBean> create(@RequestBody SignUpBean up) {
        return ResponseEntity.ok(agentService.create(up));
    }

    @PutMapping("/edit/{id}")
    public ResponseEntity<AgentBean> update(@RequestBody UpdatePasswordBean passwordBean, @PathVariable Integer id) {
        return ResponseEntity.ok(agentService.update(passwordBean, id));
    }

    @DeleteMapping("{id}")
    public ResponseEntity<AgentBean> delete(@PathVariable Integer id) {
        return ResponseEntity.ok(agentService.delete(id));
    }
}
