package com.bytesw.bytebot.controller.agency;

import com.bytesw.bytebot.bean.agency.FlightBean;
import com.bytesw.bytebot.bean.agency.FlightSearchBean;
import com.bytesw.bytebot.bean.agency.FlightSearchRequestBean;
import com.bytesw.bytebot.service.agency.FlightService;
import com.bytesw.xdf.controller.XDFController;
import com.bytesw.xdf.sql.beans.Pagination;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/service/flights")
public class FlightController extends XDFController<FlightBean, Integer> {

    @Autowired
    private GsonBuilder gsonBuilder;

    @Autowired
    private FlightService flightService;

    public FlightController(FlightService service) {
        super(service);
    }

    @Override
    @PostMapping("/page")
    public ResponseEntity<String> findAllByPagination(@RequestBody Pagination<FlightBean> pagination) {
        return super.findAllByPagination(pagination);
    }

    @Override
    @PostMapping("/create")
    public ResponseEntity<String> create(@RequestBody FlightBean flightBean, HttpServletRequest req) {
        Gson gson = gsonBuilder.create();
        flightBean = (FlightBean) this.service.create(flightBean);
        return new ResponseEntity(gson.toJson(flightBean), HttpStatus.CREATED);
    }

    @Override
    @PutMapping("/edit/{id}")
    public ResponseEntity<String> update(@PathVariable Integer id, @RequestBody FlightBean flightBean, HttpServletRequest req) {
        FlightBean flightResponse = (FlightBean) service.update(flightBean, id);
        return new ResponseEntity<>(gson.toJson(flightResponse), HttpStatus.OK);
    }

    @Override
    @DeleteMapping("{id}")
    public ResponseEntity<String> delete(@PathVariable Integer id, HttpServletRequest req) {
        Gson gson = gsonBuilder.create();
        Map<String, String> resp = new HashMap<>();
        try {
            service.delete(id);
            resp.put("message", "OK");
            return new ResponseEntity<>(gson.toJson(resp), HttpStatus.OK);
        } catch (Exception e) {
            resp.put("message", "ERROR");
            return new ResponseEntity<>(gson.toJson(resp), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    @GetMapping("{id}")
    public ResponseEntity<String> findOne(@PathVariable Integer id, HttpServletRequest req) {
        FlightBean flightBean = (FlightBean) service.getById(id);
        return new ResponseEntity(gson.toJson(flightBean), HttpStatus.OK);
    }
}
