package com.bytesw.bytebot.controller.agency;

import com.bytesw.bytebot.bean.agency.PassengerBean;
import com.bytesw.bytebot.bean.agency.PassengerMapBean;
import com.bytesw.bytebot.bean.agency.SignUpBean;
import com.bytesw.bytebot.bean.agency.UpdatePasswordBean;
import com.bytesw.bytebot.model.agency.User;
import com.bytesw.bytebot.model.agency.enums.RoleEnum;
import com.bytesw.bytebot.service.agency.PassengerService;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import com.bytesw.xdf.sql.beans.Pagination;
import com.google.gson.GsonBuilder;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@CrossOrigin
@RequestMapping("/service/passengers")
public class PassengerController {

    private final PassengerService passengerService;

    public PassengerController(PassengerService passengerService) {
        this.passengerService = passengerService;
    }

    @PostMapping("/page")
    public CustomPaginationBean<PassengerBean> getByPagination(@RequestBody CustomPaginationBean<PassengerBean> pagination) {
        return passengerService.getByPagination(pagination);
    }

    @GetMapping("{id}")
    public ResponseEntity<PassengerBean> getById(@PathVariable Integer id) {
        return ResponseEntity.ok(passengerService.getById(id));
    }

    @PostMapping("/create")
    public ResponseEntity<PassengerBean> create(@RequestBody SignUpBean up) {
        return ResponseEntity.ok(passengerService.create(up));
    }

    @PutMapping("/edit/{id}")
    public ResponseEntity<PassengerBean> update(@RequestBody UpdatePasswordBean passwordBean, @PathVariable Integer id) {
        return ResponseEntity.ok(passengerService.update(passwordBean, id));
    }

    @DeleteMapping("{id}")
    public ResponseEntity<PassengerBean> delete(@PathVariable Integer id) {
        return ResponseEntity.ok(passengerService.delete(id));
    }

    @GetMapping
    public ResponseEntity<List<PassengerMapBean>> getAll() {
        return ResponseEntity.ok(passengerService.getAll());
    }
}
