package com.bytesw.bytebot.controller.agency;

import com.bytesw.bytebot.bean.agency.FlightSearchBean;
import com.bytesw.bytebot.bean.agency.FlightSearchRequestBean;
import com.bytesw.bytebot.bean.agency.ReservationBean;
import com.bytesw.bytebot.bean.agency.UpdateReservationStatus;
import com.bytesw.bytebot.service.agency.FlightService;
import com.bytesw.bytebot.service.agency.ReservationService;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import jdk.jfr.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.List;

@RestController
@RequestMapping("/service/reservations")
public class ReservationController {

    @Autowired
    private ReservationService reservationService;

    @Autowired
    private FlightService flightService;

    @PostMapping("/page")
    public ResponseEntity<CustomPaginationBean<ReservationBean>> getByPaginationAgent(@RequestBody CustomPaginationBean<ReservationBean> pagination) {
        return ResponseEntity.ok(reservationService.getByPaginationAgent(pagination));
    }

    @PostMapping("/create")
    public ResponseEntity<ReservationBean> create(@RequestBody ReservationBean reservationBean) {
        return ResponseEntity.status(HttpStatus.CREATED).body(reservationService.create(reservationBean));
    }

    @PutMapping("/edit/{id}")
    public ResponseEntity<ReservationBean> update(@RequestBody UpdateReservationStatus updateReservationStatus, @PathVariable Integer id) {
        return ResponseEntity.ok(reservationService.updateStatusReservation(updateReservationStatus, id));
    }

    @DeleteMapping("{id}")
    public ResponseEntity<ReservationBean> delete(@PathVariable Integer id) {
        return ResponseEntity.ok(reservationService.delete(id));
    }

    @GetMapping("{id}")
    public ResponseEntity<ReservationBean> getById(@PathVariable Integer id) {
        return ResponseEntity.ok(reservationService.getById(id));
    }

    @GetMapping("/flights/search")
    public ResponseEntity<List<FlightSearchBean>> searchFlights(@PathParam("airportOriginId") Integer airportOriginId,
                                                                @PathParam("airportDestinationId") Integer airportDestinationId,
                                                                @PathParam("departureDate") String departureDate) {
        return ResponseEntity.ok(flightService.searchFlights(airportOriginId, airportDestinationId, departureDate));
    }
}
