package com.bytesw.bytebot.controller.agency;

import com.bytesw.bytebot.bean.agency.SignUpBean;
import com.bytesw.bytebot.bean.agency.SupervisorBean;
import com.bytesw.bytebot.bean.agency.UpdatePasswordBean;
import com.bytesw.bytebot.service.agency.SupervisorService;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/service/supervisors")
public class SupervisorController {

    private final SupervisorService supervisorService;

    public SupervisorController(SupervisorService supervisorService) {
        this.supervisorService = supervisorService;
    }

    @GetMapping
    public ResponseEntity<List<SupervisorBean>> getAll() {
        return ResponseEntity.ok(supervisorService.getAll());
    }

    @GetMapping("{id}")
    public ResponseEntity<SupervisorBean> getById(@PathVariable Integer id) {
        return ResponseEntity.ok(supervisorService.getById(id));
    }

    @PostMapping("/page")
    public ResponseEntity<CustomPaginationBean<SupervisorBean>> getByPagination(@RequestBody CustomPaginationBean<SupervisorBean> pagination) {
        return ResponseEntity.ok(supervisorService.getByPagination(pagination));
    }

    @PostMapping("/create")
    public ResponseEntity<SupervisorBean> create(@RequestBody SignUpBean bean) {
        return ResponseEntity.ok(supervisorService.create(bean));
    }

    @PutMapping("/edit/{id}")
    public ResponseEntity<SupervisorBean> update(@RequestBody UpdatePasswordBean passwordBean, @PathVariable Integer id) {
        return ResponseEntity.ok(supervisorService.update(passwordBean, id));
    }

    @DeleteMapping("{id}")
    public ResponseEntity<SupervisorBean> delete(@PathVariable Integer id) {
        return ResponseEntity.ok(supervisorService.delete(id));
    }
}
