package com.bytesw.bytebot.model.agency;

import com.bytesw.bytebot.model.agency.converters.ReservationStatusConverter;
import com.bytesw.bytebot.model.agency.converters.TicketTypeConverter;
import com.bytesw.bytebot.model.agency.converters.TravelClassConverter;
import com.bytesw.bytebot.model.agency.enums.ReservationStatusEnum;
import com.bytesw.bytebot.model.agency.enums.TicketTypeEnum;
import com.bytesw.bytebot.model.agency.enums.TravelClassEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import net.minidev.json.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonBackReference;

import javax.persistence.*;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "ag_reservations")
@Getter
@Setter
@AllArgsConstructor
@NoArgsConstructor
public class Reservation {

    @Id
    @Column(name = "reservation_id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    @ManyToOne
    @JoinColumn(name = "passenger_id", referencedColumnName = "passenger_id")
    private Passenger passenger;

    @ManyToOne
    @JoinColumn(name = "agent_id", referencedColumnName = "agent_id")
    private AgencyAgent agencyAgent;

    @Column(name = "reservation_ticket_type")
    @Convert(converter = TicketTypeConverter.class)
    private TicketTypeEnum ticketType;

    @Column(name = "reservation_travel_class")
    @Convert(converter = TravelClassConverter.class)
    private TravelClassEnum travelClass;

    @Column(name = "reservation_status")
    @Convert(converter = ReservationStatusConverter.class)
    private ReservationStatusEnum status;

    @Column(name = "reservation_departure_date")
    private LocalDateTime departureDate;

    @OneToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "payment_id", referencedColumnName = "payment_id")
    private Payment payment;

    @ManyToOne
    @JoinColumn(name = "flight_id", referencedColumnName = "flight_id")
    private Flight flight;

    @Column(name = "created_date")
    private LocalDate createdDate;

    @Column(name = "reservation_deleted_at")
    private LocalDate deletedAt;
}
