package com.bytesw.bytebot.service.agency;

import com.bytesw.bytebot.bean.agency.AgentBean;
import com.bytesw.bytebot.bean.agency.SignUpBean;
import com.bytesw.bytebot.bean.agency.UpdatePasswordBean;
import com.bytesw.bytebot.model.agency.AgencyAgent;
import com.bytesw.bytebot.model.agency.Passenger;
import com.bytesw.bytebot.model.agency.Supervisor;
import com.bytesw.bytebot.model.agency.User;
import com.bytesw.bytebot.model.agency.enums.RoleEnum;
import com.bytesw.bytebot.repository.agency.AgencyAgentRepository;
import com.bytesw.bytebot.repository.agency.AgencyUserRepository;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class AgencyAgentService {

    private final AgencyAgentRepository agentRepository;
    private final AgencyUserRepository userRepository;

    public AgencyAgentService(AgencyAgentRepository agentRepository, AgencyUserRepository userRepository) {
        this.agentRepository = agentRepository;
        this.userRepository = userRepository;
    }

    public List<AgentBean> getAll() {
        List<AgencyAgent> agents = agentRepository.findAllByDeletedAtIsNull();

        List<AgentBean> agentBeans = new ArrayList<>();
        for (AgencyAgent agent: agents) {
            agentBeans.add(AgentBean.miniClone(agent));
        }
        return agentBeans;
    }

    public CustomPaginationBean<AgentBean> getByPagination(CustomPaginationBean<AgentBean> pagination) {
        Pageable pageable = CustomPaginationBean.buildPageable(pagination);

        Specification<AgencyAgent> specification = (Specification<AgencyAgent>) (root, query, criteriaBuilder)
                -> criteriaBuilder.isNull(root.get("deletedAt"));

        Page<AgencyAgent> page = agentRepository.findAll(specification, pageable);

        Page<AgentBean> beanPage = convertPage(page);
        return CustomPaginationBean.fromPage(beanPage);
    }

    public Page<AgentBean> convertPage(Page<AgencyAgent> originalPage) {
        List<AgentBean> agentBeans = originalPage.getContent().stream()
                .map(AgentBean::miniClone)
                .collect(Collectors.toList());

        return new PageImpl<>(agentBeans, originalPage.getPageable(), originalPage.getTotalElements());
    }

    public AgentBean getByUserId(Integer userId) {
        Optional<AgencyAgent> agent = agentRepository.findByUser_Id(userId);

        if (!agent.isPresent())
            throw new RuntimeException("Agent not found!");

        return AgentBean.miniClone(agent.get());
    }

    public AgentBean getById(Integer id) {
        Optional<AgencyAgent> agent = agentRepository.findByIdAndDeletedAtIsNull(id);

        if (!agent.isPresent())
            throw new RuntimeException("Agent not found!");

        return AgentBean.miniClone(agent.get());
    }

    public AgentBean create(SignUpBean up) {
        AgencyAgent agent = new AgencyAgent();
        User user = new User();

        user.setEmail(up.getEmail());
        user.setPassword(up.getPassword());
        user.setRole(RoleEnum.AGENT);

        agent.setName(up.getName());
        agent.setLastname(up.getLastname());
        agent.setDetails(up.getDetails());
        agent.setUser(user);
        return AgentBean.miniClone(agentRepository.save(agent));
    }

    public AgentBean update(UpdatePasswordBean passwordBean, Integer id) {
        Optional<AgencyAgent> agent = agentRepository.findByIdAndDeletedAtIsNull(id);

        if (!agent.isPresent())
            throw new RuntimeException("Agent not found!");

        agent.get().getUser().setPassword(passwordBean.getPassword());

        return AgentBean.miniClone(agentRepository.save(agent.get()));
    }

    public AgentBean delete(Integer id) {
        Optional<AgencyAgent> agent = agentRepository.findByIdAndDeletedAtIsNull(id);

        if (!agent.isPresent())
            throw new RuntimeException("Agent not found!");

        agent.get().setDeletedAt(LocalDate.now());

        return AgentBean.miniClone(agentRepository.save(agent.get()));
    }
}
