package com.bytesw.bytebot.service.agency;

import com.bytesw.bytebot.bean.agency.*;
import com.bytesw.bytebot.model.agency.AgencyAgent;
import com.bytesw.bytebot.model.agency.Flight;
import com.bytesw.bytebot.model.agency.Passenger;
import com.bytesw.bytebot.model.agency.User;
import com.bytesw.bytebot.model.agency.enums.RoleEnum;
import com.bytesw.bytebot.repository.agency.AgencyUserRepository;
import com.bytesw.bytebot.repository.agency.PassengerRepository;
import com.bytesw.bytebot.service.CustomPaginationService;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import com.bytesw.xdf.exception.NotFoundException;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class PassengerService extends CustomPaginationService<Passenger> {

    private final PassengerRepository passengerRepository;
    private final AgencyUserRepository userRepository;

    public PassengerService(PassengerRepository passengerRepository, AgencyUserRepository userRepository) {
        this.passengerRepository = passengerRepository;
        this.userRepository = userRepository;
    }


    public CustomPaginationBean<PassengerBean> getByPagination(CustomPaginationBean<PassengerBean> pagination) {
        Pageable pageable = CustomPaginationBean.buildPageable(pagination);

        Specification<Passenger> specification = (Specification<Passenger>) (root, query, criteriaBuilder)
                -> criteriaBuilder.isNull(root.get("deletedAt"));

        Page<Passenger> page = passengerRepository.findAll(specification, pageable);

        Page<PassengerBean> passengerBeans = convertPage(page);

        return CustomPaginationBean.fromPage(passengerBeans);
    }

    public Page<PassengerBean> convertPage(Page<Passenger> originalPage) {
        List<PassengerBean> passengerBeans = originalPage.getContent().stream()
                .map(PassengerBean::miniClone)
                .collect(Collectors.toList());

        return new PageImpl<>(passengerBeans, originalPage.getPageable(), originalPage.getTotalElements());
    }

    public PassengerBean getById(Integer id) {
        Optional<Passenger> passenger = passengerRepository.findByIdAndDeletedAtIsNull(id);

        if (!passenger.isPresent())
            throw new RuntimeException("Passenger not found!");

        return PassengerBean.miniClone(passenger.get());
    }

    public List<PassengerMapBean> getAll() {
        Iterable<Passenger> passengers = passengerRepository.findAllByDeletedAtIsNull();

        List<PassengerMapBean> passengerBeans = new ArrayList<>();

        for (Passenger passenger: passengers) {
            passengerBeans.add(PassengerMapBean.miniClone(passenger));
        }

        return passengerBeans;
    }

    public PassengerBean getByUserId(Integer userId) {
        Optional<Passenger> passenger = passengerRepository.findByUser_Id(userId);

        if (!passenger.isPresent())
            throw new RuntimeException("Passenger not found!");

        return PassengerBean.miniClone(passenger.get());
    }

    public PassengerBean create(SignUpBean up) {
        Passenger passenger = new Passenger();
        User user = new User();

        user.setEmail(up.getEmail());
        user.setPassword(up.getPassword());
        user.setRole(RoleEnum.PASSENGER);

        passenger.setName(up.getName());
        passenger.setLastname(up.getLastname());
        passenger.setCity(up.getCity());
        passenger.setPhone(up.getPhone());
        passenger.setAddress(up.getAddress());
        passenger.setCity(up.getCity());
        passenger.setCountry(up.getCountry());
        passenger.setUser(user);


        return PassengerBean.miniClone(passengerRepository.save(passenger));
    }

    public PassengerBean update(UpdatePasswordBean passwordBean, Integer id) {
        Optional<Passenger> passenger = passengerRepository.findByIdAndDeletedAtIsNull(id);

        if (!passenger.isPresent())
            throw new RuntimeException("Agent not found!");

        Passenger passengerBd = passenger.get();
        passengerBd.getUser().setPassword(passwordBean.getPassword());

        return PassengerBean.miniClone(passengerRepository.save(passengerBd));
    }

    public PassengerBean delete(Integer id) {
        Optional<Passenger> passenger = this.passengerRepository.findByIdAndDeletedAtIsNull(id);

        if (!passenger.isPresent())
            throw new NotFoundException(String.format("Passenger with id %d not found!", id));

        Passenger passengerDelete = passenger.get();
        passengerDelete.setDeletedAt(LocalDate.now());

        return PassengerBean.miniClone(this.passengerRepository.save(passengerDelete));
    }
}
