package com.bytesw.bytebot.service.agency;

import com.bytesw.bytebot.bean.agency.AgentBean;
import com.bytesw.bytebot.bean.agency.SignUpBean;
import com.bytesw.bytebot.bean.agency.SupervisorBean;
import com.bytesw.bytebot.bean.agency.UpdatePasswordBean;
import com.bytesw.bytebot.model.agency.Passenger;
import com.bytesw.bytebot.model.agency.Supervisor;
import com.bytesw.bytebot.model.agency.User;
import com.bytesw.bytebot.model.agency.enums.RoleEnum;
import com.bytesw.bytebot.repository.agency.AgencyUserRepository;
import com.bytesw.bytebot.repository.agency.SupervisorRepository;
import com.bytesw.bytebot.utils.CustomPaginationBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class SupervisorService {

    private final SupervisorRepository supervisorRepository;
    private final AgencyUserRepository userRepository;

    public SupervisorService(SupervisorRepository supervisorRepository, AgencyUserRepository userRepository) {
        this.supervisorRepository = supervisorRepository;
        this.userRepository = userRepository;
    }

    public List<SupervisorBean> getAll() {
        List<Supervisor> supervisors = supervisorRepository.findAllByDeletedAtIsNull();

        List<SupervisorBean> supervisorBeans = new ArrayList<>();
        for (Supervisor supervisor: supervisors) {
            supervisorBeans.add(SupervisorBean.miniClone(supervisor));
        }
        return supervisorBeans;
    }

    public CustomPaginationBean<SupervisorBean> getByPagination(CustomPaginationBean<SupervisorBean> pagination) {
        Pageable pageable = CustomPaginationBean.buildPageable(pagination);
        Specification<Supervisor> specification = (Specification<Supervisor>) (root, query, criteriaBuilder)
                -> criteriaBuilder.isNull(root.get("deletedAt"));

        Page<Supervisor> page = supervisorRepository.findAll(specification, pageable);

        Page<SupervisorBean> supervisorBeans = convertPage(page);

        return CustomPaginationBean.fromPage(supervisorBeans);
    }

    public Page<SupervisorBean> convertPage(Page<Supervisor> originalPage) {
        List<SupervisorBean> supervisorBeans = originalPage.getContent().stream()
                .map(SupervisorBean::miniClone)
                .collect(Collectors.toList());

        return new PageImpl<>(supervisorBeans, originalPage.getPageable(), originalPage.getTotalElements());
    }

    public SupervisorBean getByUserId(Integer userId) {
        Optional<Supervisor> supervisor = supervisorRepository.findByUser_Id(userId);

        if (!supervisor.isPresent())
            throw new RuntimeException("Supervisor not found!");
        return SupervisorBean.miniClone(supervisor.get());
    }

    public SupervisorBean getById(Integer id) {
        Optional<Supervisor> supervisor = supervisorRepository.findById(id);

        if (!supervisor.isPresent())
            throw new RuntimeException("Supervisor not found!");

        return SupervisorBean.miniClone(supervisor.get());
    }

    public SupervisorBean create(SignUpBean up) {
        Supervisor supervisor = new Supervisor();
        User user = new User();

        user.setEmail(up.getEmail());
        user.setPassword(up.getPassword());
        user.setRole(RoleEnum.SUPERVISOR);

        supervisor.setName(up.getName());
        supervisor.setLastname(up.getLastname());
        supervisor.setDetails(up.getDetails());
        supervisor.setUser(user);
        return SupervisorBean.miniClone(supervisorRepository.save(supervisor));
    }

    public SupervisorBean update(UpdatePasswordBean passwordBean, Integer id) {
        Optional<Supervisor> supervisorOp = supervisorRepository.findById(id);

        if (!supervisorOp.isPresent())
            throw new RuntimeException("Supervisor not found!");

        Supervisor supervisor = supervisorOp.get();
        supervisor.getUser().setPassword(passwordBean.getPassword());

        return SupervisorBean.miniClone(supervisorRepository.save(supervisor));
    }

    public SupervisorBean delete(Integer id) {
        Optional<Supervisor> supervisors = supervisorRepository.findByIdAndDeletedAtIsNull(id);

        if (!supervisors.isPresent())
            throw new RuntimeException("Supervisor not found!");

        Supervisor supervisor = supervisors.get();
        supervisor.setDeletedAt(LocalDate.now());

        userRepository.save(supervisor.getUser());

        return SupervisorBean.miniClone(supervisor);
    }
}
