package com.bytesw.bytebot.service.agency;

import com.bytesw.bytebot.bean.agency.*;
import com.bytesw.bytebot.model.agency.Passenger;
import com.bytesw.bytebot.model.agency.User;
import com.bytesw.bytebot.model.agency.enums.RoleEnum;
import com.bytesw.bytebot.repository.agency.AgencyUserRepository;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.Optional;

@Service
@Transactional
public class UserService {

    private final AgencyUserRepository agencyUserRepository;
    private final AgencyAgentService agencyAgentService;
    private final PassengerService passengerService;
    private final SupervisorService supervisorService;

    public UserService(AgencyUserRepository agencyUserRepository, AgencyAgentService agencyAgentService, PassengerService passengerService, SupervisorService supervisorService) {
        this.agencyUserRepository = agencyUserRepository;
        this.agencyAgentService = agencyAgentService;
        this.passengerService = passengerService;
        this.supervisorService = supervisorService;
    }

    public ResponseEntity<?> signIn(SignInBean in) {
        Optional<User> user =
                agencyUserRepository.findByEmailAndPassword(in.getEmail(), in.getPassword());

        if (!user.isPresent())
            throw new RuntimeException("Email y/o password incorrect!");

        RoleEnum role = user.get().getRole();

        switch (role) {
            case PASSENGER:
                PassengerBean passengerBean = passengerService.getByUserId(user.get().getId());
                return ResponseEntity.status(HttpStatus.OK).body(passengerBean);
            case AGENT:
                AgentBean agentBean = agencyAgentService.getByUserId(user.get().getId());
                return ResponseEntity.status(HttpStatus.OK).body(agentBean);
            case SUPERVISOR:
                SupervisorBean supervisorBean = supervisorService.getByUserId(user.get().getId());
                return ResponseEntity.status(HttpStatus.OK).body(supervisorBean);
        }

        throw new RuntimeException("An error occurred white save user");
    }

    public ResponseEntity<?> signUp(SignUpBean up) {

        RoleEnum role = RoleEnum.fromString(up.getRole().toUpperCase());

        switch (role) {
            case AGENT:
                AgentBean agentBean = agencyAgentService.create(up);
                return ResponseEntity.status(HttpStatus.CREATED).body(agentBean);
            case PASSENGER:
                PassengerBean passengerBean = passengerService.create(up);
                return ResponseEntity.status(HttpStatus.CREATED).body(passengerBean);
            case SUPERVISOR:
                SupervisorBean supervisorBean = supervisorService.create(up);
                return ResponseEntity.status(HttpStatus.CREATED).body(supervisorBean);

        }

        throw new RuntimeException("Error occurred white try to save user");
    }
}
