package com.bytesw.bytebot.utils;

import com.google.gson.annotations.Expose;
import lombok.Getter;
import lombok.Setter;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
public class CustomPaginationBean<T> {
    @Expose
    private int currentPage;

    @Expose
    private List<T> data;

    @Expose
    private int itemsPerPage;

    @Expose
    private List<SortFieldBean> sortFields;

    @Expose
    private int totalItems;

    @Expose
    private int totalPages;

    public static <T> Pageable buildPageable(CustomPaginationBean<T> originPagination) {
        int currentPage = originPagination.getCurrentPage();
        int itemsPerPage = originPagination.getItemsPerPage();
        List<SortFieldBean> sortFields = originPagination.getSortFields();

        Sort sort = null;
        if (sortFields != null && !sortFields.isEmpty()) {
            sort = Sort.by(sortFields.stream()
                    .map(sf -> sf.getDirection().equals("desc") ? Sort.Order.desc(sf.getField()) : Sort.Order.asc(sf.getField()))
                    .toArray(Sort.Order[]::new));
            return PageRequest.of(currentPage, itemsPerPage, sort);
        }
        return PageRequest.of(currentPage, itemsPerPage);
    }

    public static <T> CustomPaginationBean<T> fromPage(Page<T> page) {
        CustomPaginationBean<T> customPagination = new CustomPaginationBean<>();
        customPagination.setCurrentPage(page.getNumber());
        customPagination.setItemsPerPage(page.getSize());
        customPagination.setTotalItems((int) page.getTotalElements());
        customPagination.setTotalPages(page.getTotalPages());
        customPagination.setData(page.getContent());

        if (page.getPageable().getSort() == null) {
            customPagination.setSortFields(new ArrayList<>());
        }
        else {
            List<SortFieldBean> sorts = page.getPageable().getSort().stream().map(sort -> {
                SortFieldBean sortBean = new SortFieldBean();
                sortBean.setDirection(sort.getDirection().name());
                sortBean.setField(sort.getProperty());
                return sortBean;
            }).collect(Collectors.toList());
            customPagination.setSortFields(sorts);
        }

        return customPagination;
    }

}


